@@ -7,7 +7,7 @@ use Mite::Miteception -all;
 extends qw(Mite::Role);
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.006007';
+our $VERSION   = '0.006008';
 
 use Path::Tiny;
 use mro;
@@ -17,7 +17,7 @@ use B ();
 has extends =>
   is            => bare,
   accessor      => 'superclasses',
-  isa           => ArrayRef[Str],
+  isa           => ArrayRef[NonEmptyStr],
   default       => sub { [] },
   trigger       => sub {
       my $self = shift;
@@ -182,6 +182,7 @@ sub compilation_stages {
         _compile_package
         _compile_uses_mite
         _compile_pragmas
+        _compile_load_storable
         _compile_imported_functions
         _compile_extends
         _compile_with
@@ -195,6 +196,17 @@ sub compilation_stages {
     );
 }
 
+sub _compile_load_storable {
+    my $self = shift;
+
+    return unless
+        grep   { defined($_) and $_ eq true }
+        map    { $_->cloner_method }
+        values %{ $self->all_attributes };
+
+    return "use Storable ();\n";
+}
+
 sub _compile_extends {
     my $self = shift;
 
