@@ -64,30 +64,40 @@
 
         # Attribute: name
         croak "Missing key in constructor: name" unless exists $args->{"name"};
-        do {
+        (
+            (
+                do {
 
-            package Mite::Shim;
-            defined( $args->{"name"} ) and do {
-                ref( \$args->{"name"} ) eq 'SCALAR'
-                  or ref( \( my $val = $args->{"name"} ) ) eq 'SCALAR';
-            }
-          }
+                    package Mite::Shim;
+                    defined( $args->{"name"} ) and do {
+                        ref( \$args->{"name"} ) eq 'SCALAR'
+                          or ref( \( my $val = $args->{"name"} ) ) eq 'SCALAR';
+                    }
+                }
+            ) && do { package Mite::Shim; length( $args->{"name"} ) > 0 }
+          )
           or croak "Type check failed in constructor: %s should be %s", "name",
-          "Str";
+          "NonEmptyStr";
         $self->{"name"} = $args->{"name"};
 
         # Attribute: shim_name
         if ( exists $args->{"shim_name"} ) {
-            do {
+            (
+                (
+                    do {
 
-                package Mite::Shim;
-                defined( $args->{"shim_name"} ) and do {
-                    ref( \$args->{"shim_name"} ) eq 'SCALAR'
-                      or ref( \( my $val = $args->{"shim_name"} ) ) eq 'SCALAR';
-                }
-              }
+                        package Mite::Shim;
+                        defined( $args->{"shim_name"} ) and do {
+                            ref( \$args->{"shim_name"} ) eq 'SCALAR'
+                              or ref( \( my $val = $args->{"shim_name"} ) ) eq
+                              'SCALAR';
+                        }
+                    }
+                )
+                  && do { package Mite::Shim; length( $args->{"shim_name"} ) > 0 }
+              )
               or croak "Type check failed in constructor: %s should be %s",
-              "shim_name", "Str";
+              "shim_name", "NonEmptyStr";
             $self->{"shim_name"} = $args->{"shim_name"};
         }
 
@@ -149,20 +159,27 @@
                 ( ref($value) eq 'HASH' ) and do {
                     my $ok = 1;
                     for my $i ( values %{$value} ) {
-                        ( $ok = 0, last ) unless do {
+                        ( $ok = 0, last )
+                          unless (
+                            (
+                                do {
 
-                            package Mite::Shim;
-                            defined($i) and do {
-                                ref( \$i ) eq 'SCALAR'
-                                  or ref( \( my $val = $i ) ) eq 'SCALAR';
-                            }
-                        }
+                                    package Mite::Shim;
+                                    defined($i) and do {
+                                        ref( \$i ) eq 'SCALAR'
+                                          or ref( \( my $val = $i ) ) eq
+                                          'SCALAR';
+                                    }
+                                }
+                            )
+                            && ( length($i) > 0 )
+                          );
                     };
                     $ok;
                 }
               }
               or croak "Type check failed in constructor: %s should be %s",
-              "imported_functions", "HashRef[Str]";
+              "imported_functions", "HashRef[NonEmptyStr]";
             $self->{"imported_functions"} = $value;
         };
 
@@ -178,20 +195,27 @@
                 ( ref($value) eq 'ARRAY' ) and do {
                     my $ok = 1;
                     for my $i ( @{$value} ) {
-                        ( $ok = 0, last ) unless do {
+                        ( $ok = 0, last )
+                          unless (
+                            (
+                                do {
 
-                            package Mite::Shim;
-                            defined($i) and do {
-                                ref( \$i ) eq 'SCALAR'
-                                  or ref( \( my $val = $i ) ) eq 'SCALAR';
-                            }
-                        }
+                                    package Mite::Shim;
+                                    defined($i) and do {
+                                        ref( \$i ) eq 'SCALAR'
+                                          or ref( \( my $val = $i ) ) eq
+                                          'SCALAR';
+                                    }
+                                }
+                            )
+                            && ( length($i) > 0 )
+                          );
                     };
                     $ok;
                 }
               }
               or croak "Type check failed in constructor: %s should be %s",
-              "required_methods", "ArrayRef[Str]";
+              "required_methods", "ArrayRef[NonEmptyStr]";
             $self->{"required_methods"} = $value;
         };
 
@@ -351,16 +375,24 @@
     sub shim_name {
         @_ > 1
           ? do {
-            do {
+            (
+                (
+                    do {
 
-                package Mite::Shim;
-                defined( $_[1] ) and do {
-                    ref( \$_[1] ) eq 'SCALAR'
-                      or ref( \( my $val = $_[1] ) ) eq 'SCALAR';
-                }
-              }
-              or croak( "Type check failed in %s: value should be %s",
-                "accessor", "Str" );
+                        package Mite::Shim;
+                        defined( $_[1] ) and do {
+                            ref( \$_[1] ) eq 'SCALAR'
+                              or ref( \( my $val = $_[1] ) ) eq 'SCALAR';
+                        }
+                    }
+                )
+                  && ( length( $_[1] ) > 0 )
+              )
+              or croak(
+                "Type check failed in %s: value should be %s",
+                "accessor",
+                "NonEmptyStr"
+              );
             $_[0]{"shim_name"} = $_[1];
             $_[0];
           }
@@ -369,18 +401,26 @@
                 exists( $_[0]{"shim_name"} ) ? $_[0]{"shim_name"} : (
                     $_[0]{"shim_name"} = do {
                         my $default_value = $_[0]->_build_shim_name;
-                        do {
+                        (
+                            (
+                                do {
 
-                            package Mite::Shim;
-                            defined($default_value) and do {
-                                ref( \$default_value ) eq 'SCALAR'
-                                  or ref( \( my $val = $default_value ) ) eq
-                                  'SCALAR';
-                            }
-                          }
+                                    package Mite::Shim;
+                                    defined($default_value) and do {
+                                        ref( \$default_value ) eq 'SCALAR'
+                                          or
+                                          ref( \( my $val = $default_value ) )
+                                          eq 'SCALAR';
+                                    }
+                                }
+                            )
+                              && ( length($default_value) > 0 )
+                          )
                           or croak(
                             "Type check failed in default: %s should be %s",
-                            "shim_name", "Str" );
+                            "shim_name",
+                            "NonEmptyStr"
+                          );
                         $default_value;
                     }
                 )
